@@ -28,6 +28,7 @@ import shutil
 import subprocess
 import sys
 import tempfile
+import time
 from nose.tools import eq_, with_setup
 
 import vagrant
@@ -39,6 +40,9 @@ MULTIVM_VAGRANTFILE = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                    'vagrantfiles', 'multivm_Vagrantfile')
 VM_VAGRANTFILE = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                    'vagrantfiles', 'vm_Vagrantfile')
+SHELL_PROVISION_VAGRANTFILE = os.path.join(
+    os.path.dirname(os.path.abspath(__file__)),
+    'vagrantfiles', 'shell_provision_Vagrantfile')
 # the names of the vms from the multi-vm Vagrantfile.
 VM_1 = 'web'
 VM_2 = 'db'
@@ -50,6 +54,7 @@ TEST_BOX_URL = "http://files.vagrantup.com/lucid32.box"
 TD = None
 
 
+
 def list_boxes():
     '''
     Return a list if tuples containing the box name and box provider of each
@@ -116,35 +121,29 @@ def teardown():
 
 # TEST-LEVEL SETUP AND TEARDOWN
 
+def make_setup_vm(vagrantfile=None):
+    '''
+    Make and return a function that sets up the temporary directory with a
+    Vagrantfile.  By default, use VM_VAGRANTFILE.
+    vagrantfile: path to a vagrantfile to use as Vagrantfile in the testing temporary directory.
+    '''
+    if vagrantfile is None:
+        vagrantfile = VM_VAGRANTFILE
+
+    def setup_vm():
+        shutil.copy(vagrantfile, os.path.join(TD, 'Vagrantfile'))
+
+    return setup_vm
 
-def setup_vm():
-    shutil.copy(VM_VAGRANTFILE, os.path.join(TD, 'Vagrantfile'))
 
 def teardown_vm():
     '''
-    Destroys the VM after each test method.
-    It is not an error if the VM has already been destroyed.
+    Attempts to destroy every VM in the Vagrantfile in the temporary directory, TD.
+    It is not an error if a VM has already been destroyed.
     '''
-    try:
-        subprocess.check_call(
-            'vagrant destroy -f'.format(TEST_BOX_NAME),
-            cwd=TD, shell=True)
-    except subprocess.CalledProcessError:
-        pass
-    finally:
-        # remove Vagrantfile created by setup 'vagrant init' command.
-        os.unlink(os.path.join(TD, "Vagrantfile"))
-
-
-def setup_multivm():
-    shutil.copy(MULTIVM_VAGRANTFILE, os.path.join(TD, 'Vagrantfile'))
-
-
-def teardown_multivm():
     try:
         # Try to destroy any vagrant box that might be running.
-        subprocess.check_call('vagrant destroy -f',
-                              cwd=TD, shell=True)
+        subprocess.check_call('vagrant destroy -f', cwd=TD, shell=True)
     except subprocess.CalledProcessError:
         pass
     finally:
@@ -152,7 +151,8 @@ def teardown_multivm():
         os.unlink(os.path.join(TD, "Vagrantfile"))
 
 
-@with_setup(setup_vm, teardown_vm)
+
+@with_setup(make_setup_vm(), teardown_vm)
 def test_plugin_list_parsing():
     '''
     Test the parsing the output of the `vagrant plugin list` command.
@@ -161,34 +161,35 @@ def test_plugin_list_parsing():
     listing = '''sahara (0.0.16)
 vagrant-login (1.0.1, system)
 '''
-    goal = [{'name': 'sahara', 'version': '0.0.16', 'system': False},
-            {'name': 'vagrant-login', 'version': '1.0.1', 'system': True}]
+    # Can compare tuples to Plugin class b/c Plugin is a collections.namedtuple.
+    goal = [('sahara', '0.0.16', False), ('vagrant-login', '1.0.1', True)]
     v = vagrant.Vagrant(TD)
     parsed = [v._parse_plugin_list_line(l) for l in listing.splitlines()]
     assert goal == parsed, 'The parsing of the test listing did not match the goal.\nlisting={!r}\ngoal={!r}\nparsed_listing={!r}'.format(listing, goal, parsed)
 
 
-@with_setup(setup_vm, teardown_vm)
+@with_setup(make_setup_vm(), teardown_vm)
 def test_vm_status():
     '''
-    Test whether vagrant.status() correctly reports state of the VM.
+    Test whether vagrant.status() correctly reports state of the VM, in a
+    single-VM environment.
     '''
     v = vagrant.Vagrant(TD)
-    assert v.NOT_CREATED in v.status().values(), "Before going up status should be vagrant.NOT_CREATED"
+    assert v.NOT_CREATED == v.status()[0].state, "Before going up status should be vagrant.NOT_CREATED"
     command = 'vagrant up'
     subprocess.check_call(command, cwd=TD, shell=True)
-    assert v.RUNNING in v.status().values(), "After going up status should be vagrant.RUNNING"
+    assert v.RUNNING in v.status()[0].state, "After going up status should be vagrant.RUNNING"
 
     command = 'vagrant halt'
     subprocess.check_call(command, cwd=TD, shell=True)
-    assert v.POWEROFF in v.status().values(), "After halting status should be vagrant.POWEROFF"
+    assert v.POWEROFF in v.status()[0].state, "After halting status should be vagrant.POWEROFF"
 
     command = 'vagrant destroy -f'
     subprocess.check_call(command, cwd=TD, shell=True)
-    assert v.NOT_CREATED in v.status().values(), "After destroying status should be vagrant.NOT_CREATED"
+    assert v.NOT_CREATED in v.status()[0].state, "After destroying status should be vagrant.NOT_CREATED"
 
 
-@with_setup(setup_vm, teardown_vm)
+@with_setup(make_setup_vm(), teardown_vm)
 def test_vm_lifecycle():
     '''
     Test methods controlling the VM - init(), up(), halt(), destroy().
@@ -199,22 +200,22 @@ def test_vm_lifecycle():
     # Test init by removing Vagrantfile, since v.init() will create one.
     os.unlink(os.path.join(TD, 'Vagrantfile'))
     v.init(TEST_BOX_NAME)
-    assert v.NOT_CREATED in v.status().values()
+    assert v.NOT_CREATED == v.status()[0].state
 
     v.up()
-    assert v.RUNNING in v.status().values()
+    assert v.RUNNING == v.status()[0].state
 
     v.suspend()
-    assert v.SAVED in v.status().values()
+    assert v.SAVED == v.status()[0].state
 
     v.halt()
-    assert v.POWEROFF in v.status().values()
+    assert v.POWEROFF == v.status()[0].state
 
     v.destroy()
-    assert v.NOT_CREATED in v.status().values()
+    assert v.NOT_CREATED == v.status()[0].state
 
 
-@with_setup(setup_vm, teardown_vm)
+@with_setup(make_setup_vm(), teardown_vm)
 def test_vm_config():
     '''
     Test methods retrieving ssh config settings, like user, hostname, and port.
@@ -250,7 +251,7 @@ def test_vm_config():
     eq_(keyfile, parsed_config["IdentityFile"])
 
 
-@with_setup(setup_vm, teardown_vm)
+@with_setup(make_setup_vm(), teardown_vm)
 def test_vm_sandbox_mode():
     '''
     Test methods for enabling/disabling the sandbox mode
@@ -260,7 +261,6 @@ def test_vm_sandbox_mode():
     '''
     # Only test Sahara if it is installed.
     # This leaves the testing of Sahara to people who care.
-    # These tests are broken?  Does SandboxVagrant work?
     sahara_installed = _plugin_installed(vagrant.Vagrant(TD), 'sahara')
     if not sahara_installed:
         return
@@ -268,28 +268,28 @@ def test_vm_sandbox_mode():
     v = vagrant.SandboxVagrant(TD)
 
     sandbox_status = v.sandbox_status()
-    assert "unknown" in v.status().values(), "Before the VM goes up the status should be 'unknown', " + "got:'{}'".format(sandbox_status)
+    assert sandbox_status == "unknown", "Before the VM goes up the status should be 'unknown', " + "got:'{}'".format(sandbox_status)
 
     v.up()
     sandbox_status = v.sandbox_status()
-    assert "off" in v.status().values(), "After the VM goes up the status should be 'off', " + "got:'{}'".format(sandbox_status)
+    assert sandbox_status == "off", "After the VM goes up the status should be 'off', " + "got:'{}'".format(sandbox_status)
 
     v.sandbox_on()
     sandbox_status = v.sandbox_status()
-    assert "on" in v.status().values(), "After enabling the sandbox mode the status should be 'on', " + "got:'{}'".format(sandbox_status)
+    assert sandbox_status == "on", "After enabling the sandbox mode the status should be 'on', " + "got:'{}'".format(sandbox_status)
 
     v.sandbox_off()
     sandbox_status = v.sandbox_status()
-    assert "off" in v.status().values(), "After disabling the sandbox mode the status should be 'off', " + "got:'{}'".format(sandbox_status)
+    assert sandbox_status == "off", "After disabling the sandbox mode the status should be 'off', " + "got:'{}'".format(sandbox_status)
 
     v.sandbox_on()
     v.halt()
     sandbox_status = v.sandbox_status()
-    assert "on" in v.status().values(), "After halting the VM the status should be 'on', " + "got:'{}'".format(sandbox_status)
+    assert sandbox_status == "on", "After halting the VM the status should be 'on', " + "got:'{}'".format(sandbox_status)
 
     v.up()
     sandbox_status = v.sandbox_status()
-    assert "on" in v.status().values(), "After bringing the VM up again the status should be 'on', " + "got:'{}'".format(sandbox_status)
+    assert sandbox_status == "on", "After bringing the VM up again the status should be 'on', " + "got:'{}'".format(sandbox_status)
 
     test_file_contents = _read_test_file(v)
     print test_file_contents
@@ -301,6 +301,8 @@ def test_vm_sandbox_mode():
     eq_(test_file_contents, "foo", "The test file should read 'foo'")
 
     v.sandbox_rollback()
+    time.sleep(10)  # https://github.com/jedi4ever/sahara/issues/16
+
     test_file_contents = _read_test_file(v)
     print test_file_contents
     eq_(test_file_contents, None, "There should be no test file")
@@ -316,6 +318,8 @@ def test_vm_sandbox_mode():
     eq_(test_file_contents, "bar", "The test file should read 'bar'")
 
     v.sandbox_rollback()
+    time.sleep(10)  # https://github.com/jedi4ever/sahara/issues/16
+
     test_file_contents = _read_test_file(v)
     print test_file_contents
     eq_(test_file_contents, "foo", "The test file should read 'foo'")
@@ -327,10 +331,10 @@ def test_vm_sandbox_mode():
 
     v.destroy()
     sandbox_status = v.sandbox_status()
-    assert "unknown" in v.status().values(), "After destroying the VM the status should be 'unknown', " + "got:'{}'".format(sandbox_status)
+    assert sandbox_status == "unknown", "After destroying the VM the status should be 'unknown', " + "got:'{}'".format(sandbox_status)
 
 
-@with_setup(setup_vm, teardown_vm)
+@with_setup(make_setup_vm(), teardown_vm)
 def test_boxes():
     '''
     Test methods for manipulating boxes - adding, listing, removing.
@@ -356,16 +360,9 @@ def test_boxes():
     eq_((box_name, provider) in boxes, True,
         "There should be a dummy box in the list of boxes")
 
-    # Test listing box names
-    box_names = list_box_names()
-    reported_box_names = v.box_list()
-    for box in box_names:
-        eq_(box in reported_box_names, True,
-            "The box '{}' should be in the list returned by box_list()".format(box))
-
-    # Test listing box names and providers
+    # Test listing boxes
     boxes = list_boxes()
-    reported_boxes = v.box_list_long()
+    reported_boxes = [(b.name, b.provider) for b in v.box_list()]
     for box in boxes:
         eq_(box in reported_boxes, True,
             "The box '{}' should be in the list returned by box_list()".format(box))
@@ -377,7 +374,7 @@ def test_boxes():
         "There should be no dummy box after it's been deleted")
 
 
-@with_setup(setup_vm, teardown_vm)
+@with_setup(make_setup_vm(SHELL_PROVISION_VAGRANTFILE), teardown_vm)
 def test_provisioning():
     '''
     Test provisioning support.  The tested provision config creates a file on
@@ -395,47 +392,46 @@ def test_provisioning():
     eq_(test_file_contents, "foo", "The test file should contain 'foo'")
 
 
-@with_setup(setup_multivm, teardown_multivm)
+@with_setup(make_setup_vm(MULTIVM_VAGRANTFILE), teardown_vm)
 def test_multivm_lifecycle():
     v = vagrant.Vagrant(TD)
 
     # test getting multiple statuses at once
-    statuses = v.status()
-    eq_(statuses[VM_1], v.NOT_CREATED)
-    eq_(statuses[VM_2], v.NOT_CREATED)
+    eq_(v.status(VM_1)[0].state, v.NOT_CREATED)
+    eq_(v.status(VM_2)[0].state, v.NOT_CREATED)
 
     v.up(vm_name=VM_1)
-    eq_(v.status()[VM_1], v.RUNNING)
-    eq_(v.status()[VM_2], v.NOT_CREATED)
+    eq_(v.status(VM_1)[0].state, v.RUNNING)
+    eq_(v.status(VM_2)[0].state, v.NOT_CREATED)
 
     # start both vms
     v.up()
-    eq_(v.status()[VM_1], v.RUNNING)
-    eq_(v.status()[VM_2], v.RUNNING)
+    eq_(v.status(VM_1)[0].state, v.RUNNING)
+    eq_(v.status(VM_2)[0].state, v.RUNNING)
 
     v.halt(vm_name=VM_1)
-    eq_(v.status()[VM_1], v.POWEROFF)
-    eq_(v.status()[VM_2], v.RUNNING)
+    eq_(v.status(VM_1)[0].state, v.POWEROFF)
+    eq_(v.status(VM_2)[0].state, v.RUNNING)
 
     v.destroy(vm_name=VM_1)
-    eq_(v.status()[VM_1], v.NOT_CREATED)
-    eq_(v.status()[VM_2], v.RUNNING)
+    eq_(v.status(VM_1)[0].state, v.NOT_CREATED)
+    eq_(v.status(VM_2)[0].state, v.RUNNING)
 
     v.suspend(vm_name=VM_2)
-    eq_(v.status()[VM_1], v.NOT_CREATED)
-    eq_(v.status()[VM_2], v.SAVED)
+    eq_(v.status(VM_1)[0].state, v.NOT_CREATED)
+    eq_(v.status(VM_2)[0].state, v.SAVED)
 
     v.destroy(vm_name=VM_2)
-    eq_(v.status()[VM_1], v.NOT_CREATED)
-    eq_(v.status()[VM_2], v.NOT_CREATED)
+    eq_(v.status(VM_1)[0].state, v.NOT_CREATED)
+    eq_(v.status(VM_2)[0].state, v.NOT_CREATED)
 
 
-@with_setup(setup_multivm, teardown_multivm)
+@with_setup(make_setup_vm(MULTIVM_VAGRANTFILE), teardown_vm)
 def test_multivm_config():
     '''
     Test methods retrieving configuration settings.
     '''
-    v = vagrant.Vagrant(TD)
+    v = vagrant.Vagrant(TD, quiet_stdout=False, quiet_stderr=False)
     v.up(vm_name=VM_1)
     command = "vagrant ssh-config " + VM_1
     ssh_config = subprocess.check_output(command, cwd=TD, shell=True)
@@ -471,8 +467,13 @@ def _execute_command_in_vm(v, command):
     Run command via ssh on the test vagrant box.  Returns a tuple of the
     return code and output of the command.
     '''
+    vagrant_exe = vagrant.get_vagrant_executable()
+
+    if not vagrant_exe:
+        raise RuntimeError(vagrant.VAGRANT_NOT_FOUND_WARNING)
+
     # ignore the fact that this host is not in our known hosts
-    ssh_command = [vagrant.VAGRANT_EXE, 'ssh', '-c', command]
+    ssh_command = [vagrant_exe, 'ssh', '-c', command]
     return subprocess.check_output(ssh_command, cwd=v.root)
 
 
@@ -499,4 +500,4 @@ def _read_test_file(v):
 
 def _plugin_installed(v, plugin_name):
     plugins = v.plugin_list()
-    return plugin_name in [plugin['name'] for plugin in plugins]
+    return plugin_name in [plugin.name for plugin in plugins]
