@@ -6,40 +6,28 @@ with the `vagrant` CLI without the boilerplate (and errors) of calling
 The API attempts to conform closely to the API of the `vagrant` command line,
 including method names and parameter names.
 
-Quick and dirty test to up, look at, and destroy (!) a Vagrant box.  Run from
-the directory holding your Vagrantfile.
-
-python -c 'import vagrant
-v = vagrant.Vagrant()
-v.up()
-print v.status()
-print v.user()
-print v.hostname()
-print v.port()
-print v.keyfile()
-print v.user_hostname()
-print v.user_hostname_port()
-print v.conf()
-v.destroy();
-'
-
-For unit tests, see tests/test_vagrant.py
-
-Dependencies:
-
-- `vagrant` should be installed and in your PATH.
-
+Documentation of usage, testing, installation, etc., can be found at
+https://github.com/todddeluca/python-vagrant.
 '''
 
+import collections
 import os
 import re
 import subprocess
 import sys
+import logging
 
 
 # python package version
 # should match r"^__version__ = '(?P<version>[^']+)'$" for setup.py
-__version__ = '0.4.5'
+__version__ = '0.5.0'
+
+
+log = logging.getLogger(__name__)
+
+
+VAGRANT_NOT_FOUND_WARNING = 'The Vagrant executable cannot be found. ' \
+                            'Please check if it is in the system path.'
 
 
 def which(program):
@@ -99,9 +87,18 @@ def which(program):
 
 
 # The full path to the vagrant executable, e.g. '/usr/bin/vagrant'
-VAGRANT_EXE = which('vagrant')
-assert VAGRANT_EXE, ('The Vagrant executable cannot be found. ' +
-                     'Please check if it is in the system path.')
+def get_vagrant_executable():
+    return which('vagrant')
+
+
+if get_vagrant_executable() is None:
+    log.warn(VAGRANT_NOT_FOUND_WARNING)
+
+
+# Classes for listings of Statuses, Boxes, and Plugins
+Status = collections.namedtuple('Status', ['name', 'state', 'provider'])
+Box = collections.namedtuple('Box', ['name', 'provider', 'version'])
+Plugin = collections.namedtuple('Plugin', ['name', 'version', 'system'])
 
 
 class Vagrant(object):
@@ -112,14 +109,12 @@ class Vagrant(object):
 
     Works by using the `vagrant` executable and a `Vagrantfile`.
     '''
-
     # statuses
     RUNNING = 'running'  # vagrant up
     NOT_CREATED = 'not created'  # vagrant destroy
     POWEROFF = 'poweroff'  # vagrant halt
     ABORTED = 'aborted'  # The VM is in an aborted state
     SAVED = 'saved' # vagrant suspend
-
     # LXC statuses
     STOPPED = 'stopped'
     FROZEN = 'frozen'
@@ -134,26 +129,33 @@ class Vagrant(object):
         'ubuntu-precise64': 'http://files.vagrantup.com/precise64.box',
     }
 
-    def __init__(self, root=None):
+    def __init__(self, root=None, quiet_stdout=True, quiet_stderr=True):
         '''
         root: a directory containing a file named Vagrantfile.  Defaults to
         os.getcwd(). This is the directory and Vagrantfile that the Vagrant
         instance will operate on.
+        quiet_stdout: If True, the stdout of vagrant commands whose output is
+          not captured for further processing will be sent to devnull.
+        quiet_stderr: If True, the stderr of vagrant commands whose output is
+          not captured for further processing will be sent to devnull.
         '''
         self.root = os.path.abspath(root) if root is not None else os.getcwd()
         self._cached_conf = {}
+        self._vagrant_exe = None # cache vagrant executable path
+        self.quiet_stdout = quiet_stdout
+        self.quiet_stderr = quiet_stderr
 
     def version(self):
         '''
         Return the installed vagrant version, as a string, e.g. '1.5.0'
         '''
-        output = self._run_vagrant_command('--version')
+        output = self._run_vagrant_command(['--version'])
         m = re.search(r'^Vagrant (?P<version>.+)$', output)
         if m is None:
             raise Exception('Failed to parse vagrant --version output. output={!r}'.format(output))
         return m.group('version')
 
-    def init(self, box_name=None, box_url=None, **kwargs):
+    def init(self, box_name=None, box_url=None):
         '''
         From the Vagrant docs:
 
@@ -167,10 +169,10 @@ class Vagrant(object):
 
         Note: if box_url is given, box_name should also be given.
         '''
-        self._run_vagrant_command('init', box_name, box_url, **kwargs)
+        self._call_vagrant_command(['init', box_name, box_url])
 
     def up(self, no_provision=False, provider=None, vm_name=None,
-           provision=None, provision_with=None, **kwargs):
+           provision=None, provision_with=None):
         '''
         Launch the Vagrant box.
         provision_with: optional list of provisioners to enable.
@@ -192,13 +194,9 @@ class Vagrant(object):
         no_provision_arg = '--no-provision' if no_provision else None
         provision_arg = None if provision is None else '--provision' if provision else '--no-provision'
 
-        self._run_vagrant_command('up',
-                                  vm_name,
-                                  no_provision_arg,
-                                  provision_arg,
-                                  provider_arg,
-                                  prov_with_arg, providers_arg,
-                                  **kwargs)
+        self._call_vagrant_command(['up', vm_name, no_provision_arg,
+                                   provision_arg, provider_arg,
+                                   prov_with_arg, providers_arg])
         try:
             self.conf(vm_name=vm_name)  # cache configuration
         except subprocess.CalledProcessError:
@@ -206,7 +204,7 @@ class Vagrant(object):
             # however vm_name is required for conf() or ssh_config().
             pass
 
-    def provision(self, vm_name=None, provision_with=None, **kwargs):
+    def provision(self, vm_name=None, provision_with=None):
         '''
         Runs the provisioners defined in the Vagrantfile.
         vm_name: optional VM name string.
@@ -215,11 +213,10 @@ class Vagrant(object):
         '''
         prov_with_arg = None if provision_with is None else '--provision-with'
         providers_arg = None if provision_with is None else ','.join(provision_with)
-        self._run_vagrant_command('provision', vm_name, prov_with_arg,
-                                  providers_arg, **kwargs)
+        self._call_vagrant_command(['provision', vm_name, prov_with_arg,
+                                   providers_arg])
 
-    def reload(self, vm_name=None, provision=None,
-               provision_with=None, **kwargs):
+    def reload(self, vm_name=None, provision=None, provision_with=None):
         '''
         Quoting from Vagrant docs:
         > The equivalent of running a halt followed by an up.
@@ -236,42 +233,57 @@ class Vagrant(object):
         prov_with_arg = None if provision_with is None else '--provision-with'
         providers_arg = None if provision_with is None else ','.join(provision_with)
         provision_arg = None if provision is None else '--provision' if provision else '--no-provision'
-        self._run_vagrant_command('reload', vm_name, provision_arg,
-                                  prov_with_arg, providers_arg,
-                                  **kwargs)
+        self._call_vagrant_command(['reload', vm_name, provision_arg,
+                                   prov_with_arg, providers_arg])
 
-    def suspend(self, vm_name=None, **kwargs):
+    def suspend(self, vm_name=None):
         '''
         Suspend/save the machine.
         '''
-        self._run_vagrant_command('suspend', vm_name, **kwargs)
+        self._call_vagrant_command(['suspend', vm_name])
         self._cached_conf[vm_name] = None  # remove cached configuration
 
-    def halt(self, vm_name=None, force=False, **kwargs):
+    def halt(self, vm_name=None, force=False):
         '''
         Halt the Vagrant box.
 
         force: If True, force shut down.
         '''
         force_opt = '--force' if force else None
-        self._run_vagrant_command('halt', vm_name, force_opt, **kwargs)
+        self._call_vagrant_command(['halt', vm_name, force_opt])
         self._cached_conf[vm_name] = None  # remove cached configuration
 
-    def destroy(self, vm_name=None, **kwargs):
+    def destroy(self, vm_name=None):
         '''
         Terminate the running Vagrant box.
         '''
-        self._run_vagrant_command('destroy', vm_name, '--force', **kwargs)
+        self._call_vagrant_command(['destroy', vm_name, '--force'])
         self._cached_conf[vm_name] = None  # remove cached configuration
 
     def status(self, vm_name=None):
         '''
-        Returns a dictionary mapping Vagrant box names to statuses.  Statuses
-        are RUNNING, POWEROFF, SAVED and NOT_CREATED, corresponding to vagrant
-        up, halt, suspend, and destroy, respectively.
+        Return the results of a `vagrant status` call as a list of one or more
+        Status objects.  A Status contains the following attributes:
 
-        In a single-VM environment or when the vm_name parameter is used in
-        a multi-VM environment, a status string is returned:
+        - name: The VM name in a multi-vm environment.  'default' otherwise.
+        - state: The state of the underlying guest machine (i.e. VM).
+        - provider: the name of the VM provider, e.g. 'virtualbox'.  None
+          if no provider is output by vagrant.
+
+        This information corresponds with the current return values from running
+        `vagrant status`, as of Vagrant 1.5.
+
+        Example return values for a multi-VM environment:
+
+            [Status(name='web', state='not created', provider='virtualbox'),
+             Status(name='db', state='not created', provider='virtualbox')]
+
+        And for a single-VM environment:
+
+            [Status(name='default', state='not created', provider='virtualbox')]
+
+        Possible states include, but are not limited to (since new states are
+        being added as Vagrant evolves):
 
         - 'not created' if the vm is destroyed
         - 'running' if the vm is up
@@ -284,59 +296,71 @@ class Vagrant(object):
         virtualbox and vmware_fusion) and has started adding the provider in
         the status string, like 'not created (virtualbox)'.
 
-        There might be other statuses, but the Vagrant docs were unclear.
+        Implementation notes:
 
-        When vm_name is not given in a multi-VM environment a dictionary
-        mapping vm names to statuses will be returned.
+        - The human-readable output of vagrant lists the vm name as 'default'
+          in a single vm environment.  This is in contrast to the
+          Machine-readable output from vagrant, which lists the vm name
+          (a.k.a. target) as '' in a single VM environment.
+        - The human readable states differ from machine readable states.  For
+          example, 'not created' versus 'not_created'.  In order to future-proof
+          code using python-vagrant, use the status constants defined in the
+          Vagrant class instead of hardcoding the string.  At some point
+          parsing will switch from using the human-readable output to the
+          machine readable output and the state values might change as well.
         '''
-        # example output:
+        # example output (without provdier):
         # Current VM states:
         #
         # default                  poweroff
         #
         # The VM is powered off. To restart the VM, simply run `vagrant up`
 
-        # example multi-VM environment output:
-        # Current VM states:
+        # example multi-VM environment output (with provider):
+        # Current machine states:
         #
-        # web                      running
-        # db                       running
+        # web                       not created (virtualbox)
+        # db                        not created (virtualbox)
         #
         # This environment represents multiple VMs. The VMs are all listed
         # above with their current state. For more information about a specific
         # VM, run `vagrant status NAME`.
 
-        # Example status lines, showing the vm name, status, and provider.
-        # default                  not created (virtualbox)
-        # default                  saved (virtualbox)
-        # default                  running (virtualbox)
-        # default                  poweroff (virtualbox)
-
-        output = self._run_vagrant_command('status', vm_name)
+        output = self._run_vagrant_command(['status', vm_name])
         # The format of output is expected to be a
         #   - "Current VM states:" line (vagrant 1)
         #   - "Current machine states" line (vagrant 1.1)
         # followed by a blank line, followed by one or more status lines,
         # followed by a blank line.
-        statuses = {}
-        state = 1 # looking for for the 'Current ... states' line
-        for line in output.splitlines():
 
-            if state == 1 and re.search('^Current (VM|machine) states:', line.strip()):
-                state = 2 # looking for the blank line
-            elif state == 2 and not line.strip():
-                state = 3 # looking for machine status lines
-            elif state == 3 and line.strip():
-                vm_name_and_status, provider = self._parse_provider_line(line)
-                # Split vm_name from status.  Only works for recognized statuses.
-                m = re.search(r'^(?P<vm_name>.*?)\s+(?P<status>' +
+        # Parsing the output of `vagrant status`
+        # Currently parsing is constrained to known states.  Otherwise how
+        # could we know where the VM name ends and the state begins.
+        # Once --machine-readable output is stable (a work in progress as of
+        # Vagrant 1.5), this constraint can be lifted.
+        START_LINE, FIRST_BLANK, VM_STATUS = 1, 2, 3
+        statuses = []
+        parse_state = START_LINE # looking for for the 'Current ... states' line
+        for line in output.splitlines():
+            line = line.strip()
+            if parse_state == START_LINE and re.search('^Current (VM|machine) states:', line):
+                parse_state = FIRST_BLANK # looking for the first blank line
+            elif parse_state == FIRST_BLANK and not line:
+                parse_state = VM_STATUS # looking for machine status lines
+            elif parse_state == VM_STATUS and line:
+                vm_name_and_state, provider = self._parse_provider_line(line)
+                # Split vm_name from status.  Only works for recognized states.
+                m = re.search(r'^(?P<vm_name>.*?)\s+(?P<state>' +
                               '|'.join(self.STATUSES) + ')$',
-                              vm_name_and_status)
+                              vm_name_and_state)
                 if not m:
-                    raise Exception('ParseError: Failed to properly parse vm name and status from line.', line, output)
+                    raise Exception('ParseError: Failed to properly parse vm name and status from line.',
+                                    line, output)
                 else:
-                    statuses[m.group('vm_name')] = m.group('status')
-            elif state == 3 and not line.strip():
+                    status = Status(m.group('vm_name'), m.group('state'), provider)
+                    statuses.append(status)
+            elif parse_state == VM_STATUS and not line:
+                # Found the second blank line.  All done.
                 break
 
         return statuses
@@ -392,7 +416,7 @@ class Vagrant(object):
                 IdentitiesOnly yes
         '''
         # capture ssh configuration from vagrant
-        return self._run_vagrant_command('ssh-config', vm_name)
+        return self._run_vagrant_command(['ssh-config', vm_name])
 
     def user(self, vm_name=None):
         '''
@@ -469,7 +493,7 @@ class Vagrant(object):
         port_suffix = ':' + port if port else ''
         return user_prefix + self.hostname(vm_name=vm_name) + port_suffix
 
-    def box_add(self, name, url, provider=None, force=False, **kwargs):
+    def box_add(self, name, url, provider=None, force=False):
         '''
         Adds a box with given name, from given url.
 
@@ -480,49 +504,64 @@ class Vagrant(object):
         if provider is not None:
             cmd += ['--provider', provider]
 
-        self._run_vagrant_command(*cmd, **kwargs)
+        self._call_vagrant_command(cmd)
 
     def box_list(self):
         '''
-        Returns a list of all available box names.  For example:
+        Run `vagrant box list` and return a list of Box objects containing the
+        results.  A Box object has the following attributes:
 
-            ['precise32', 'precise64']
+        - name: the box-name.
+        - provider: the box-provider.
+        - version: the box-version.
+
+        Example output:
+
+            [Box(name='precise32', provider='virtualbox', version=None),
+             Box(name='precise64', provider='virtualbox', version=None),
+             Box(name='trusty64', provider='virtualbox', version=None)]
+
+        Implementation Notes:
+
+        - The box-version is not currently returned, since we parse the
+          human-readable vagrant output, where it is not listed.  As of Vagrant
+          1.5 (or 1.4?) is is available in the --machine-readable output.
+          Once parsing is switched to use that output, it will be available.
+        - As of Vagrant >= 1.1, boxes are listed with names and providers.
         '''
-        return [name for name, provider in self.box_list_long()]
-
-    def box_list_long(self):
-        '''
-        Returns a list of all available boxes as tuples containing the box
-        name and provider.  For example:
-
-            [('precise32', 'virtualbox'), ('precise64', 'virtualbox')]
-
-        As of Vagrant >= 1.1, boxes are listed with names and providers.
-        '''
-        output = self._run_vagrant_command('box', 'list')
+        output = self._run_vagrant_command(['box', 'list'])
         boxes = []
         for line in output.splitlines():
             name, provider = self._parse_provider_line(line)
-            boxes.append((name, provider))
+            box = Box(name, provider, version=None) # not currently parsing the box version
+            boxes.append(box)
         return boxes
 
-    def box_remove(self, name, provider, **kwargs):
+    def box_remove(self, name, provider):
         '''
         Removes the box matching name and provider. It is an error if no box
         matches name and provider.
         '''
-        self._run_vagrant_command('box', 'remove', name, provider, **kwargs)
+        self._call_vagrant_command(['box', 'remove', name, provider])
 
     def plugin_list(self):
         '''
-        Return a list of dicts containing the following information about
-        installed plugins:
-        - 'name': The plugin name, as a string.
-        - 'version': The plugin version, as a string.
-        - 'system': A boolean, presumably indicating whether this plugin is a
-          "core" part of vagrant, though the feature is undocumented.
+        Return a list of Plugin objects containing the following information
+        about installed plugins:
+
+        - name: The plugin name, as a string.
+        - version: The plugin version, as a string.
+        - system: A boolean, presumably indicating whether this plugin is a
+          "core" part of vagrant, though the feature is not yet documented
+          in the Vagrant 1.5 docs.
+
+        Example output:
+
+            [Plugin(name='sahara', version='0.0.16', system=False),
+             Plugin(name='vagrant-login', version='1.0.1', system=True),
+             Plugin(name='vagrant-share', version='1.0.1', system=True)]
         '''
-        output = self._run_vagrant_command('plugin', 'list')
+        output = self._run_vagrant_command(['plugin', 'list'])
         return [self._parse_plugin_list_line(l) for l in output.splitlines()]
 
     def _parse_plugin_list_line(self, line):
@@ -537,9 +576,7 @@ class Vagrant(object):
         if m is None:
             raise Exception('Error parsing plugin listing line.', line)
         else:
-            listing = m.groupdict()
-            listing['system'] = bool(listing['system'])
-            return listing
+            return Plugin(m.group('name'), m.group('version'), bool(m.group('system')))
 
     def _parse_provider_line(self, line):
         '''
@@ -605,79 +642,47 @@ class Vagrant(object):
             conf[key] = value.strip('"')
         return conf
 
-    def _run_vagrant_command(self, *args, **kwargs):
-        '''
-        args: A tuple of arguments to a vagrant command line.
-        e.g. ['up', 'my_vm_name', '--no-provision'] or
-        ['up', None, '--no-provision'] for a non-Multi-VM environment.
-        kwargs:
-        - capture_output.  If False, vagrant command output will be sent
-          to stdout instead of returned from the function.  The default is
-          True.
-        - quiet_stderr:  If True, the stderr of the vagrant command will be
-          sent to devnull.
-        '''
+    def _make_vagrant_command(self, args):
+        if self._vagrant_exe is None:
+            self._vagrant_exe = get_vagrant_executable()
+
+        if not self._vagrant_exe:
+            raise RuntimeError(VAGRANT_NOT_FOUND_WARNING)
+
         # filter out None args.  Since vm_name is None in non-Multi-VM
         # environments, this quitely removes it from the arguments list
         # when it is not specified.
-        command = [VAGRANT_EXE] + [arg for arg in args if arg is not None]
+        return [self._vagrant_exe] + [arg for arg in args if arg is not None]
 
-        # Suppress stderr if quiet_stderr is True
-        if kwargs.get('quiet_stderr', False):
-            fo_stderr = open(os.devnull, 'wb')
-        else:
-            fo_stderr = sys.stderr
-
-        if not kwargs.get('capture_output', True):
-            retval = subprocess.check_call(command, cwd=self.root, stderr=fo_stderr)
-        else:
-            retval = subprocess.check_output(command, cwd=self.root, stderr=fo_stderr)
-
-        # Close what you open
-        if kwargs.get('quiet_stderr', False):
-            fo_stderr.close()
-
-        return retval
-
-    def _confirm(self, prompt=None, resp=False):
+    def _call_vagrant_command(self, args):
         '''
-        Prompts for yes or no response from the user. Returns True for yes
-        and False for no.
-
-        :prompt resp: The default value assumed by the caller when
-        user simply types ENTER.
-
-        >>> confirm(prompt='Create Directory?', resp=True)
-        Create Directory? [Y/n]:
-        True
-        >>> confirm(prompt='Create Directory?', resp=False)
-        Create Directory? [y/N]:
-        False
-        >>> confirm(prompt='Create Directory?', resp=False)
-        Create Directory? [y/N]: y
-        True
+        Run a vagrant command.  Return None.
+        args: A sequence of arguments to a vagrant command line.
 
         '''
-
-        if prompt is None:
-            prompt = 'Confirm'
-
-        if resp:
-            prompt = '%s [%s/%s]: ' % (prompt, 'Y', 'n')
+        # Make subprocess command
+        command = self._make_vagrant_command(args)
+        if self.quiet_stdout or self.quiet_stderr:
+            # Redirect stdout and/or stderr to devnull
+            # Use with stmt to close filehandle in case of exception
+            with open(os.devnull, 'wb') as fh:
+                outfh = fh if self.quiet_stdout else sys.stdout
+                errfh = fh if self.quiet_stderr else sys.stderr
+                subprocess.check_call(command, cwd=self.root,
+                                      stdout=outfh, stderr=errfh)
         else:
-            prompt = '%s [%s/%s]: ' % (prompt, 'y', 'N')
+            subprocess.check_call(command, cwd=self.root)
 
-        while True:
-            ans = raw_input(prompt)
-            if not ans:
-                return resp
-            if ans not in ['y', 'Y', 'n', 'N']:
-                print 'Please enter y or n.'
-                continue
-            if ans == 'y' or ans == 'Y':
-                return True
-            if ans == 'n' or ans == 'N':
-                return False
+    def _run_vagrant_command(self, args):
+        '''
+        Run a vagrant command and return its stdout.
+        args: A sequence of arguments to a vagrant command line.
+        e.g. ['up', 'my_vm_name', '--no-provision'] or
+        ['up', None, '--no-provision'] for a non-Multi-VM environment.
+        '''
+        # Make subprocess command
+        command = self._make_vagrant_command(args)
+        return subprocess.check_output(command, cwd=self.root)
 
 
 class SandboxVagrant(Vagrant):
@@ -685,20 +690,20 @@ class SandboxVagrant(Vagrant):
     Support for sandbox mode using the Sahara gem (https://github.com/jedi4ever/sahara).
     '''
 
-    def _run_sandbox_command(self, *args):
-        return self._run_vagrant_command('sandbox', *args)
+    def _run_sandbox_command(self, args):
+        return self._run_vagrant_command(['sandbox'] + list(args))
 
     def sandbox_commit(self, vm_name=None):
         '''
         Permanently writes all the changes made to the VM.
         '''
-        self._run_sandbox_command('commit', vm_name)
+        self._run_sandbox_command(['commit', vm_name])
 
     def sandbox_off(self, vm_name=None):
         '''
         Disables the sandbox mode.
         '''
-        self._run_sandbox_command('off', vm_name)
+        self._run_sandbox_command(['off', vm_name])
 
     def sandbox_on(self, vm_name=None):
         '''
@@ -707,13 +712,13 @@ class SandboxVagrant(Vagrant):
         This requires the Sahara gem to be installed
         (https://github.com/jedi4ever/sahara).
         '''
-        self._run_sandbox_command('on', vm_name)
+        self._run_sandbox_command(['on', vm_name])
 
     def sandbox_rollback(self, vm_name=None):
         '''
         Reverts all the changes made to the VM since the last commit.
         '''
-        self._run_sandbox_command('rollback', vm_name)
+        self._run_sandbox_command(['rollback', vm_name])
 
     def sandbox_status(self, vm_name=None):
         '''
@@ -725,7 +730,7 @@ class SandboxVagrant(Vagrant):
         - unknown
         - not installed
         '''
-        vagrant_sandbox_output = self._run_sandbox_command('status', vm_name)
+        vagrant_sandbox_output = self._run_sandbox_command(['status', vm_name])
         return self._parse_vagrant_sandbox_status(vagrant_sandbox_output)
 
     def _parse_vagrant_sandbox_status(self, vagrant_output):
